@@ -1,5 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -777,24 +778,61 @@ namespace xt
 
     namespace detail
     {
-        template <class S, class I>
+        template <layout_type L>
+        struct next_idx_impl;
+
+        template <>
+        struct next_idx_impl<layout_type::row_major>
+        {
+            template <class S, class I>
+            inline auto operator()(const S& shape, I& idx)
+            {
+                for (std::size_t j = shape.size(); j > 0; --j)
+                {
+                    std::size_t i = j - 1;
+                    if (idx[i] >= shape[i] - 1)
+                    {
+                        idx[i] = 0;
+                    }
+                    else
+                    {
+                        idx[i]++;
+                        return idx;
+                    }
+                }
+                // return empty index, happens at last iteration step, but remains unused
+                return I();
+            }
+        };
+
+        template <>
+        struct next_idx_impl<layout_type::column_major>
+        {
+            template <class S, class I>
+            inline auto operator()(const S& shape, I& idx)
+            {
+                for(std::size_t i = 0; i < shape.size(); ++i)
+                {
+                    if(idx[i] >= shape[i] - 1)
+                    {
+                        idx[i] = 0;
+                    }
+                    else
+                    {
+                        idx[i]++;
+                        return idx;
+                    }
+                }
+                // return empty index, happens at last iteration step, but remains unused
+                return I();
+            }
+        };
+
+        template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL, class S, class I>
         inline auto next_idx(const S& shape, I& idx)
         {
-            for (std::size_t j = shape.size(); j > 0; --j)
-            {
-                std::size_t i = j - 1;
-                if (idx[i] >= shape[i] - 1)
-                {
-                    idx[i] = 0;
-                }
-                else
-                {
-                    idx[i]++;
-                    return idx;
-                }
-            }
-            // return empty index, happens at last iteration step, but remains unused
-            return I();
+            next_idx_impl<L> nii;
+            return nii(shape, idx);
         }
     }
 
@@ -863,12 +901,13 @@ namespace xt
      * @ingroup logical_operators
      * @brief return vector of indices where arr is not zero
      *
+     * @tparam L the traversal order
      * @param arr input array
      * @return vector of index_types where arr is not equal to zero (use `xt::from_indices` to convert)
      *
      * @sa xt::from_indices
      */
-    template <class T>
+    template <layout_type L = XTENSOR_DEFAULT_TRAVERSAL, class T>
     inline auto argwhere(const T& arr)
     {
         auto shape = arr.shape();
@@ -879,7 +918,7 @@ namespace xt
         std::vector<index_type> indices;
 
         size_type total_size = compute_size(shape);
-        for (size_type i = 0; i < total_size; i++, detail::next_idx(shape, idx))
+        for (size_type i = 0; i < total_size; i++, detail::next_idx<L>(shape, idx))
         {
             if (arr.element(std::begin(idx), std::end(idx)))
             {
