@@ -1,5 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -224,6 +225,44 @@ namespace xt
         }
     }
 
+    template <class F, class E, class R,
+              XTL_REQUIRES(std::is_convertible<typename E::value_type, typename R::value_type>)>
+    inline void copy_to_reduced(F&, const E& e, R& result)
+    {
+        if (e.layout() == layout_type::row_major)
+        {
+            std::copy(e.template cbegin<layout_type::row_major>(),
+                      e.template cend<layout_type::row_major>(),
+                      result.data());
+        }
+        else
+        {
+            std::copy(e.template cbegin<layout_type::column_major>(),
+                      e.template cend<layout_type::column_major>(),
+                      result.data());
+        }
+    }
+
+    template <class F, class E, class R,
+              XTL_REQUIRES(xtl::negation<std::is_convertible<typename E::value_type, typename R::value_type>>)>
+    inline void copy_to_reduced(F& f, const E& e, R& result)
+    {
+        if (e.layout() == layout_type::row_major)
+        {
+            std::transform(e.template cbegin<layout_type::row_major>(),
+                           e.template cend<layout_type::row_major>(),
+                           result.data(),
+                           f);
+        }
+        else
+        {
+            std::transform(e.template cbegin<layout_type::column_major>(),
+                           e.template cend<layout_type::column_major>(),
+                           result.data(),
+                           f);
+        }
+    }
+
     template <class F, class E, class X, class O>
     inline auto reduce_immediate(F&& f, E&& e, X&& axes, O&& raw_options)
     {
@@ -236,23 +275,33 @@ namespace xt
         options_t options(raw_options);
 
         using shape_type = typename xreducer_shape_type<typename std::decay_t<E>::shape_type, std::decay_t<X>, typename options_t::keep_dims>::type;
-
-        // retrieve functors from triple struct
-        auto reduce_fct = std::get<0>(f);
-        auto init_fct = std::get<1>(f);
-        auto merge_fct = std::get<2>(f);
-
-        shape_type result_shape{};
-
-        dynamic_shape<std::size_t> iter_shape = xtl::forward_sequence<dynamic_shape<std::size_t>, decltype(e.shape())>(e.shape());
-        dynamic_shape<std::size_t> iter_strides(e.dimension());
-
         using result_container_type = typename detail::xtype_for_shape<shape_type>::template type<result_type, std::decay_t<E>::static_layout>;
         result_container_type result;
 
-        if (!std::is_sorted(axes.cbegin(), axes.cend()))
+        // retrieve functors from triple struct
+        auto reduce_fct = xt::get<0>(f);
+        auto init_fct = xt::get<1>(f);
+        auto merge_fct = xt::get<2>(f);
+
+        if (axes.size() == 0)
         {
-            throw std::runtime_error("Reducing axes should be sorted");
+            result.resize(e.shape(), e.layout());
+            auto cpf = [&reduce_fct, &init_fct](const auto& v) {return reduce_fct(static_cast<result_type>(init_fct()), v); };
+            copy_to_reduced(cpf, e, result);
+            return result;
+        }
+
+        shape_type result_shape{};
+        dynamic_shape<std::size_t> iter_shape = xtl::forward_sequence<dynamic_shape<std::size_t>, decltype(e.shape())>(e.shape());
+        dynamic_shape<std::size_t> iter_strides(e.dimension());
+
+        // using std::less_equal is counter-intuitive, but as the standard says (24.4.5):
+        // A sequence is sorted with respect to a comparator comp if for any iterator i pointing to the sequence and any non-negative integer n
+        // such that i + n is a valid iterator pointing to an element of the sequence, comp(*(i + n), *i) == false.
+        // Therefore less_equal is required to detect duplicates.
+        if (!std::is_sorted(axes.cbegin(), axes.cend(), std::less_equal<>()))
+        {
+            throw std::runtime_error("Reducing axes should be sorted and should not contain duplicates");
         }
         if (axes.size() != 0 && axes[axes.size() - 1] > e.dimension() - 1)
         {
@@ -273,11 +322,20 @@ namespace xt
         auto strides_finder = e.strides().begin() + static_cast<std::ptrdiff_t>(leading_ax);
         // The computed strides contain "0" where the shape is 1 -- therefore find the next none-zero number
         std::size_t inner_stride = static_cast<std::size_t>(*strides_finder);
-        while (inner_stride == 0)
+        auto iter_bound = e.layout() == layout_type::row_major ? e.strides().begin() : (e.strides().end() - 1);
+        while (inner_stride == 0 && strides_finder != iter_bound)
         {
             (e.layout() == layout_type::row_major) ? --strides_finder : ++strides_finder;
             inner_stride = static_cast<std::size_t>(*strides_finder);
         }
+
+        if (inner_stride == 0)
+        {
+            auto cpf = [&reduce_fct, &init_fct](const auto& v) {return reduce_fct(static_cast<result_type>(init_fct()), v); };
+            copy_to_reduced(cpf, e, result);
+            return result;
+        }
+
         std::size_t inner_loop_size = static_cast<std::size_t>(inner_stride);
         std::size_t outer_loop_size = e.shape()[leading_ax];
 
@@ -407,7 +465,7 @@ namespace xt
             while (idx_res.first != true)
             {
                 std::transform(out, out + inner_loop_size, begin, out,
-                               [merge, &init_fct, &reduce_fct, &options](auto&& v1, auto&& v2) {
+                               [merge, &init_fct, &reduce_fct](auto&& v1, auto&& v2) {
                                     return merge ?
                                         reduce_fct(v1, v2) :
                                         // cast because return type of identity function is not upcasted
@@ -571,7 +629,7 @@ namespace xt
 
     template <class F, class CT, class X, class O>
     struct xcontainer_inner_types<xreducer<F, CT, X, O>>
-    {   
+    {
         using xexpression_type = std::decay_t<CT>;
         using reduce_functor_type = typename std::decay_t<F>::reduce_functor_type;
         using init_functor_type = typename std::decay_t<F>::init_functor_type;
@@ -614,7 +672,7 @@ namespace xt
      * @sa reduce
      */
     template <class F, class CT, class X, class O>
-    class xreducer : public xexpression<xreducer<F, CT, X, O>>,
+    class xreducer : public xsharable_expression<xreducer<F, CT, X, O>>,
                      public xconst_iterable<xreducer<F, CT, X, O>>,
                      public xaccessible<xreducer<F, CT, X, O>>,
                      public extension::xreducer_base_t<F, CT, X, O>
@@ -650,6 +708,7 @@ namespace xt
 
         using stepper = typename iterable_base::stepper;
         using const_stepper = typename iterable_base::const_stepper;
+        using bool_load_type = typename xexpression_type::bool_load_type;
 
         static constexpr layout_type static_layout = layout_type::dynamic;
         static constexpr bool contiguous_layout = false;
@@ -793,7 +852,7 @@ namespace xt
                            xtl::negation<detail::is_xreducer_functors<F>>)>
     inline auto reduce(F&& f, E&& e, X&& axes, EVS&& options = EVS())
     {
-        return reduce(make_xreducer_functor(std::forward<F>(f)), std::forward<E>(e), 
+        return reduce(make_xreducer_functor(std::forward<F>(f)), std::forward<E>(e),
                       std::forward<X>(axes), std::forward<EVS>(options));
     }
 
@@ -896,6 +955,7 @@ namespace xt
 
     private:
 
+        reference initial_value() const;
         reference aggregate(size_type dim) const;
         reference aggregate_impl(size_type dim, /*keep_dims=*/ std::false_type) const;
         reference aggregate_impl(size_type dim, /*keep_dims=*/ std::true_type) const;
@@ -1129,17 +1189,21 @@ namespace xt
     template <class Func, class CTA, class AX, class OX>
     inline xreducer<F, CT, X, O>::xreducer(Func&& func, CTA&& e, AX&& axes, OX&& options)
         : m_e(std::forward<CTA>(e))
-        , m_reduce(std::get<0>(func))
-        , m_init(std::get<1>(func))
-        , m_merge(std::get<2>(func))
+        , m_reduce(xt::get<0>(func))
+        , m_init(xt::get<1>(func))
+        , m_merge(xt::get<2>(func))
         , m_axes(std::forward<AX>(axes))
         , m_shape(xtl::make_sequence<inner_shape_type>(typename O::keep_dims() ? m_e.dimension() : m_e.dimension() - m_axes.size(), 0))
         , m_dim_mapping(xtl::make_sequence<dim_mapping_type>(typename O::keep_dims() ? m_e.dimension() : m_e.dimension() - m_axes.size(), 0))
         , m_options(std::forward<OX>(options))
     {
-        if (!std::is_sorted(m_axes.cbegin(), m_axes.cend()))
+        // using std::less_equal is counter-intuitive, but as the standard says (24.4.5):
+        // A sequence is sorted with respect to a comparator comp if for any iterator i pointing to the sequence and any non-negative integer n
+        // such that i + n is a valid iterator pointing to an element of the sequence, comp(*(i + n), *i) == false.
+        // Therefore less_equal is required to detect duplicates.
+        if (!std::is_sorted(m_axes.cbegin(), m_axes.cend(), std::less_equal<>()))
         {
-            throw std::runtime_error("Reducing axes should be sorted");
+            throw std::runtime_error("Reducing axes should be sorted and should not contain duplicates");
         }
         if (m_axes.size() != 0 && m_axes[m_axes.size() - 1] > m_e.dimension() - 1)
         {
@@ -1391,11 +1455,11 @@ namespace xt
     {
         if (dim >= m_offset)
         {
-            // Because the reducer uses `reset` to reset the non-reducing axes, 
+            // Because the reducer uses `reset` to reset the non-reducing axes,
             // we need to prevent that here for the KD case where.
             if (typename O::keep_dims() && std::binary_search(m_reducer->m_axes.begin(), m_reducer->m_axes.end(), dim - m_offset))
             {
-                // If keep dim activated, and dim is in the axes, do nothing! 
+                // If keep dim activated, and dim is in the axes, do nothing!
                 return;
             }
             m_stepper.reset(get_dim(dim - m_offset));
@@ -1410,7 +1474,7 @@ namespace xt
             // Note that for *not* KD this is not going to do anything
             if (typename O::keep_dims() && std::binary_search(m_reducer->m_axes.begin(), m_reducer->m_axes.end(), dim - m_offset))
             {
-                // If keep dim activated, and dim is in the axes, do nothing! 
+                // If keep dim activated, and dim is in the axes, do nothing!
                 return;
             }
             m_stepper.reset_back(get_dim(dim - m_offset));
@@ -1429,31 +1493,40 @@ namespace xt
         m_stepper.to_end(l);
     }
 
+    template <class F, class CT, class X, class O>
+    inline auto xreducer_stepper<F, CT, X, O>::initial_value() const -> reference
+    {
+        return O::has_initial_value ? m_reducer->m_options.initial_value : static_cast<reference>(m_reducer->m_init());
+    }
+
     template <class F, class CT, class X, class O>
     inline auto xreducer_stepper<F, CT, X, O>::aggregate(size_type dim) const -> reference
     {
-        if (m_reducer->m_e.shape().empty())
+        reference res;
+        if (m_reducer->m_e.size() == size_type(0))
         {
-            reference res =
-                m_reducer->m_reduce(O::has_initial_value ? m_reducer->m_options.initial_value : static_cast<reference>(m_reducer->m_init()),
-                                    *m_stepper);
-            return res;
+            res = initial_value();
+        }
+        else if (m_reducer->m_e.shape().empty() || m_reducer->m_axes.size() == 0)
+        {
+            res = m_reducer->m_reduce(initial_value(), *m_stepper);
         }
         else
         {
-            reference res = aggregate_impl(dim, typename O::keep_dims());
+            res = aggregate_impl(dim, typename O::keep_dims());
             if (O::has_initial_value && dim == 0)
             {
                 res = m_reducer->m_merge(m_reducer->m_options.initial_value, res);
             }
-            return res;
         }
+        return res;
     }
 
     template <class F, class CT, class X, class O>
     inline auto xreducer_stepper<F, CT, X, O>::aggregate_impl(size_type dim, std::false_type) const -> reference
     {
-        reference res;
+        // reference can be std::array, hence the {} initializer
+        reference res = {};
         size_type index = axis(dim);
         size_type size = shape(index);
         if (dim != m_reducer->m_axes.size() - 1)
@@ -1481,13 +1554,14 @@ namespace xt
     template <class F, class CT, class X, class O>
     inline auto xreducer_stepper<F, CT, X, O>::aggregate_impl(size_type dim, std::true_type) const -> reference
     {
-        reference res(0);
+        // reference can be std::array, hence the {} initializer
+        reference res = {};
         auto ax_it = std::find(m_reducer->m_axes.begin(), m_reducer->m_axes.end(), dim);
         if (ax_it != m_reducer->m_axes.end())
         {
             size_type index = dim;
             size_type size = m_reducer->m_e.shape()[index];
-            if (ax_it != m_reducer->m_axes.end() - 1)
+            if (ax_it != m_reducer->m_axes.end() - 1 && size != 0)
             {
                 res = aggregate_impl(dim + 1, typename O::keep_dims());
                 for (size_type i = 1; i != size; ++i)
