@@ -1,5 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -82,7 +83,7 @@ namespace xt
     };
 
     template <class CT>
-    class xscalar : public xexpression<xscalar<CT>>,
+    class xscalar : public xsharable_expression<xscalar<CT>>,
                     private xiterable<xscalar<CT>>,
                     private xaccessible<xscalar<CT>>,
                     public extension::xscalar_base_t<CT>
@@ -103,7 +104,8 @@ namespace xt
         using const_pointer = const value_type*;
         using size_type = typename inner_types::size_type;
         using difference_type = std::ptrdiff_t;
-        using simd_value_type = xsimd::simd_type<value_type>;
+        using simd_value_type = xt_simd::simd_type<value_type>;
+        using bool_load_type = xt::bool_load_type<value_type>;
 
         using iterable_base = xiterable<self_type>;
         using inner_shape_type = typename iterable_base::inner_shape_type;
@@ -151,6 +153,7 @@ namespace xt
 
         size_type size() const noexcept;
         const shape_type& shape() const noexcept;
+        size_type shape(size_type i) const noexcept;
         layout_type layout() const noexcept;
         using accessible_base::dimension;
         using accessible_base::shape;
@@ -280,8 +283,8 @@ namespace xt
         template <class align, class simd = simd_value_type>
         void store_simd(size_type i, const simd& e);
         template <class align, class requested_type = value_type,
-                  std::size_t N = xsimd::simd_traits<requested_type>::size>
-        xsimd::simd_return_type<value_type, requested_type>
+                  std::size_t N = xt_simd::simd_traits<requested_type>::size>
+        xt_simd::simd_return_type<value_type, requested_type>
         load_simd(size_type i) const;
 
     private:
@@ -360,6 +363,9 @@ namespace xt
         using size_type = typename storage_type::size_type;
         using difference_type = typename storage_type::difference_type;
 
+        template <class requested_type>
+        using simd_return_type = xt_simd::simd_return_type<value_type, requested_type>;
+
         xscalar_stepper(storage_type* c) noexcept;
 
         reference operator*() const noexcept;
@@ -372,10 +378,10 @@ namespace xt
         void to_begin() noexcept;
         void to_end(layout_type l) noexcept;
 
-        template <class R>
-        R step_simd();
+        template <class T>
+        simd_return_type<T> step_simd();
 
-        value_type step_leading();
+        void step_leading();
 
     private:
 
@@ -462,31 +468,28 @@ namespace xt
      * linear_begin / linear_end *
      *****************************/
 
-    namespace detail
+    template <class CT>
+    XTENSOR_CONSTEXPR_RETURN auto linear_begin(xscalar<CT>& c) noexcept -> decltype(c.dummy_begin())
     {
-        template <class CT>
-        constexpr auto linear_begin(xscalar<CT>& c) noexcept -> decltype(c.dummy_begin())
-        {
-            return c.dummy_begin();
-        }
+        return c.dummy_begin();
+    }
 
-        template <class CT>
-        constexpr auto linear_end(xscalar<CT>& c) noexcept -> decltype(c.dummy_end())
-        {
-            return c.dummy_end();
-        }
+    template <class CT>
+    XTENSOR_CONSTEXPR_RETURN auto linear_end(xscalar<CT>& c) noexcept -> decltype(c.dummy_end())
+    {
+        return c.dummy_end();
+    }
 
-        template <class CT>
-        constexpr auto linear_begin(const xscalar<CT>& c) noexcept -> decltype(c.dummy_begin())
-        {
-            return c.dummy_begin();
-        }
+    template <class CT>
+    XTENSOR_CONSTEXPR_RETURN auto linear_begin(const xscalar<CT>& c) noexcept -> decltype(c.dummy_begin())
+    {
+        return c.dummy_begin();
+    }
 
-        template <class CT>
-        constexpr auto linear_end(const xscalar<CT>& c) noexcept -> decltype(c.dummy_end())
-        {
-            return c.dummy_end();
-        }
+    template <class CT>
+    XTENSOR_CONSTEXPR_RETURN auto linear_end(const xscalar<CT>& c) noexcept -> decltype(c.dummy_end())
+    {
+        return c.dummy_end();
     }
 
     /**************************
@@ -531,6 +534,12 @@ namespace xt
         return zero_shape;
     }
 
+    template <class CT>
+    inline auto xscalar<CT>::shape(size_type) const noexcept -> size_type
+    {
+        return 0;
+    }
+
     template <class CT>
     inline layout_type xscalar<CT>::layout() const noexcept
     {
@@ -925,9 +934,9 @@ namespace xt
     template <class CT>
     template <class align, class requested_type, std::size_t N>
     inline auto xscalar<CT>::load_simd(size_type) const
-        -> xsimd::simd_return_type<value_type, requested_type>
+        -> xt_simd::simd_return_type<value_type, requested_type>
     {
-        return xsimd::set_simd<value_type, requested_type>(m_value);
+        return xt_simd::set_simd<value_type, requested_type>(m_value);
     }
 
     template <class T>
@@ -989,17 +998,15 @@ namespace xt
     }
 
     template <bool is_const, class CT>
-    template <class R>
-    inline R xscalar_stepper<is_const, CT>::step_simd()
+    template <class T>
+    inline auto xscalar_stepper<is_const, CT>::step_simd() -> simd_return_type<T>
     {
-        return R(p_c->operator()());
+        return simd_return_type<T>(p_c->operator()());
     }
 
     template <bool is_const, class CT>
-    inline auto xscalar_stepper<is_const, CT>::step_leading()
-        -> value_type
+    inline void xscalar_stepper<is_const, CT>::step_leading()
     {
-        return p_c->operator()();
     }
 
     /**********************************
