@@ -1,5 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -11,8 +12,11 @@
 
 #include <cstddef>
 #include <functional>
+#include <limits>
 #include <numeric>
 
+#include <xtl/xsequence.hpp>
+
 #include "xexception.hpp"
 #include "xshape.hpp"
 #include "xtensor_forward.hpp"
@@ -74,6 +78,9 @@ namespace xt
      * broadcast functions *
      ***********************/
 
+    template <class S, class size_type>
+    S uninitialized_shape(size_type size);
+
     template <class S1, class S2>
     bool broadcast_shape(const S1& input, S2& output);
 
@@ -349,10 +356,16 @@ namespace xt
         return detail::compute_strides<L>(shape, l, strides, &backstrides);
     }
 
-    // base_stride is the stride value used when shape is 1
+    template <class T1, class T2>
+    inline bool stride_match_condition(const T1& stride, const T2& shape, const T1& data_size, bool zero_strides)
+    {
+        return (shape == T2(1) && stride == T1(0) && zero_strides) || (stride == data_size);
+    }
+
+    // zero_strides should be true when strides are set to 0 if the corresponding dimensions are 1
     template <class shape_type, class strides_type>
     inline bool do_strides_match(const shape_type& shape, const strides_type& strides,
-                                 layout_type l, typename strides_type::value_type base_stride)
+                                 layout_type l, bool zero_strides)
     {
         using value_type = typename strides_type::value_type;
         value_type data_size = 1;
@@ -360,7 +373,7 @@ namespace xt
         {
             for (std::size_t i = strides.size(); i != 0; --i)
             {
-                if ((shape[i - 1] == 1 && strides[i - 1] != base_stride) || (shape[i - 1] != 1 && strides[i - 1] != data_size))
+                if (!stride_match_condition(strides[i - 1], shape[i - 1], data_size, zero_strides))
                 {
                     return false;
                 }
@@ -372,7 +385,7 @@ namespace xt
         {
             for (std::size_t i = 0; i < strides.size(); ++i)
             {
-                if ((shape[i] != 1 && strides[i] != data_size) || (shape[i] == 1 && strides[i] != base_stride))
+                if (!stride_match_condition(strides[i], shape[i], data_size, zero_strides))
                 {
                     return false;
                 }
@@ -490,6 +503,14 @@ namespace xt
         return ravel_from_strides(static_cast<strides_value_type>(index), strides);
     }
 
+    template <class S, class stype>
+    inline S uninitialized_shape(stype size)
+    {
+        using value_type = typename S::value_type;
+        using size_type = typename S::size_type;
+        return xtl::make_sequence<S>(static_cast<size_type>(size), std::numeric_limits<value_type>::max());
+    }
+
     template <class S1, class S2>
     inline bool broadcast_shape(const S1& input, S2& output)
     {
@@ -505,10 +526,10 @@ namespace xt
         }
         for (; input_index != 0; --input_index, --output_index)
         {
-            // First case: output = (0, 0, ...., 0)
+            // First case: output = (MAX, MAX, ...., MAX)
             // output is a new shape that has not been through
             // the broadcast process yet; broadcast is trivial
-            if (output[output_index - 1] == 0)
+            if (output[output_index - 1] == std::numeric_limits<value_type>::max())
             {
                 output[output_index - 1] = static_cast<value_type>(input[input_index - 1]);
             }
