@@ -1,5 +1,6 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -443,6 +444,30 @@ namespace xt
     template <class T>
     using get_value_type_t = typename get_value_type<T>::type;
 
+    /**********************
+     * get implementation *
+     **********************/
+
+    // When subclassing from std::tuple not all compilers are able to correctly instantiate get
+    // See here: https://stackoverflow.com/a/37188019/2528668
+    template <std::size_t I, template <typename... Args> class T, typename ...Args>
+    decltype(auto) get(T<Args...>&& v)
+    {
+      return std::get<I>(static_cast<std::tuple<Args...>&&>(v));
+    }
+
+    template <std::size_t I, template <typename... Args> class T, typename ...Args>
+    decltype(auto) get(T<Args...>& v)
+    {
+      return std::get<I>(static_cast<std::tuple<Args...>&>(v));
+    }
+
+    template <std::size_t I, template <typename... Args> class T, typename ...Args>
+    decltype(auto) get(const T<Args...> & v)
+    {
+      return std::get<I>(static_cast<const std::tuple<Args...> &>(v));
+    }
+
     /***************************
      * apply_cv implementation *
      ***************************/
@@ -707,6 +732,10 @@ namespace xt
       return !(a == b);
     }
 
+    /*****************
+     * has_assign_to *
+     *****************/
+
     template <class E1, class E2, class = void>
     struct has_assign_to : std::false_type
     {
@@ -718,6 +747,10 @@ namespace xt
     {
     };
 
+    /********************
+     * rebind_container *
+     ********************/
+
     template <class X, template <class, class> class C, class T, class A>
     struct rebind_container<X, C<T, A>>
     {
@@ -739,6 +772,10 @@ namespace xt
     };
 #endif
 
+    /********************
+     * get_strides_type *
+     ********************/
+
     template <class S>
     struct get_strides_type
     {
@@ -757,6 +794,10 @@ namespace xt
     template <class C>
     using get_strides_t = typename get_strides_type<C>::type;
 
+    /*******************
+     * inner_reference *
+     *******************/
+
     template <class ST>
     struct inner_reference
     {
